@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * ClutterContainer: Generic actor container interface.
  * Author: Emmanuele Bassi <ebassi@openedhand.com>
@@ -93,12 +93,6 @@ static void              create_child_meta  (ClutterContainer *container,
 static void              destroy_child_meta (ClutterContainer *container,
                                              ClutterActor     *actor);
 
-static void clutter_container_create_child_meta  (ClutterContainer *container,
-                                                  ClutterActor     *actor);
-static void clutter_container_destroy_child_meta (ClutterContainer *container,
-                                                  ClutterActor     *actor);
-
-
 static void
 clutter_container_base_init (gpointer g_iface)
 {
@@ -780,7 +774,7 @@ clutter_container_get_child_meta (ClutterContainer *container,
   return NULL;
 }
 
-/*
+/**
  * clutter_container_create_child_meta:
  * @container: a #ClutterContainer
  * @actor: a #ClutterActor
@@ -788,12 +782,25 @@ clutter_container_get_child_meta (ClutterContainer *container,
  * Creates the #ClutterChildMeta wrapping @actor inside the
  * @container, if the #ClutterContainerIface::child_meta_type
  * class member is not set to %G_TYPE_INVALID.
+ *
+ * This function is only useful when adding a #ClutterActor to
+ * a #ClutterContainer implementation outside of the
+ * #ClutterContainer::add() virtual function implementation.
+ *
+ * Applications should not call this function.
+ *
+ * Since: 1.2
  */
-static void
+void
 clutter_container_create_child_meta (ClutterContainer *container,
                                      ClutterActor     *actor)
 {
-  ClutterContainerIface *iface = CLUTTER_CONTAINER_GET_IFACE (container);
+  ClutterContainerIface *iface;
+
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (container));
+
+  iface = CLUTTER_CONTAINER_GET_IFACE (container);
 
   if (iface->child_meta_type == G_TYPE_INVALID)
     return;
@@ -804,19 +811,32 @@ clutter_container_create_child_meta (ClutterContainer *container,
     iface->create_child_meta (container, actor);
 }
 
-/*
+/**
  * clutter_container_destroy_child_meta:
  * @container: a #ClutterContainer
  * @actor: a #ClutterActor
  *
  * Destroys the #ClutterChildMeta wrapping @actor inside the
  * @container, if any.
+ *
+ * This function is only useful when removing a #ClutterActor to
+ * a #ClutterContainer implementation outside of the
+ * #ClutterContainer::add() virtual function implementation.
+ *
+ * Applications should not call this function.
+ *
+ * Since: 1.2
  */
-static void
+void
 clutter_container_destroy_child_meta (ClutterContainer *container,
                                       ClutterActor     *actor)
 {
-  ClutterContainerIface *iface = CLUTTER_CONTAINER_GET_IFACE (container);
+  ClutterContainerIface *iface;
+
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  iface = CLUTTER_CONTAINER_GET_IFACE (container);
 
   if (iface->child_meta_type == G_TYPE_INVALID)
     return;
@@ -1019,8 +1039,15 @@ clutter_container_child_set (ClutterContainer *container,
           break;
         }
 
+#if GLIB_CHECK_VERSION (2, 23, 2)
+      G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
+                            var_args, 0,
+                            &error);
+#else
       g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
       G_VALUE_COLLECT (&value, var_args, 0, &error);
+#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
+
       if (error)
         {
           /* we intentionally leak the GValue because it might
