@@ -77,8 +77,6 @@ _cogl_texture_2d_sliced_foreach_sub_texture_in_region (
   CoglSpanIter iter_x;
   CoglSpanIter iter_y;
 
-  g_assert (tex_2ds->gl_target == GL_TEXTURE_2D);
-
   /* Slice spans are stored in denormalized coordinates, and this is what
    * the _cogl_span_iter_* funcs expect to be given, so we scale the given
    * virtual coordinates by the texture size to denormalize.
@@ -118,9 +116,12 @@ _cogl_texture_2d_sliced_foreach_sub_texture_in_region (
       slice_ty1 = y_intersect_start - iter_y.pos;
       slice_ty2 = y_intersect_end - iter_y.pos;
 
-      /* Normalize slice texture coordinates */
-      slice_ty1 /= iter_y.span->size;
-      slice_ty2 /= iter_y.span->size;
+      if (tex_2ds->gl_target == GL_TEXTURE_2D)
+	{
+	  /* Normalize slice texture coordinates */
+	  slice_ty1 /= iter_y.span->size;
+	  slice_ty2 /= iter_y.span->size;
+	}
 
       /* Iterate the x axis of the virtual rectangle */
       for (_cogl_span_iter_begin (&iter_x,
@@ -153,15 +154,18 @@ _cogl_texture_2d_sliced_foreach_sub_texture_in_region (
           slice_tx1 = x_intersect_start - iter_x.pos;
           slice_tx2 = x_intersect_end - iter_x.pos;
 
-          /* Normalize slice texture coordinates */
-          slice_tx1 /= iter_x.span->size;
-          slice_tx2 /= iter_x.span->size;
-
 	  /* Pluck out opengl texture object for this slice */
 	  gl_handle = g_array_index (tex_2ds->slice_gl_handles, GLuint,
 				     iter_y.index * iter_x.array->len +
 				     iter_x.index);
 
+	  if (tex_2ds->gl_target == GL_TEXTURE_2D)
+	    {
+	      /* Normalize slice texture coordinates */
+	      slice_tx1 /= iter_x.span->size;
+	      slice_tx2 /= iter_x.span->size;
+	    }
+
           slice_coords[0] = slice_tx1;
           slice_coords[1] = slice_ty1;
           slice_coords[2] = slice_tx2;
@@ -1286,17 +1290,33 @@ _cogl_texture_2d_sliced_transform_coords_to_gl (CoglTexture *tex,
 #endif
 }
 
-static gboolean
+static CoglTransformResult
 _cogl_texture_2d_sliced_transform_quad_coords_to_gl (CoglTexture *tex,
                                                      float *coords)
 {
+  gboolean need_repeat = FALSE;
+  int i;
+
+  /* This is a bit lazy - in the case where the quad lies entirely
+   * within a single slice we could avoid the fallback. But that
+   * could likely lead to visual inconsistency if the fallback involves
+   * dropping layers, so this might be the right thing to do anyways.
+   */
   if (_cogl_texture_2d_sliced_is_sliced (tex))
-    return FALSE;
+    return COGL_TRANSFORM_SOFTWARE_REPEAT;
+
+  for (i = 0; i < 4; i++)
+    if (coords[i] < 0.0f || coords[i] > 1.0f)
+      need_repeat = TRUE;
+
+  if (need_repeat && !_cogl_texture_2d_sliced_can_hardware_repeat (tex))
+    return COGL_TRANSFORM_SOFTWARE_REPEAT;
 
   _cogl_texture_2d_sliced_transform_coords_to_gl (tex, coords + 0, coords + 1);
   _cogl_texture_2d_sliced_transform_coords_to_gl (tex, coords + 2, coords + 3);
 
-  return TRUE;
+  return (need_repeat
+          ? COGL_TRANSFORM_HARDWARE_REPEAT : COGL_TRANSFORM_NO_REPEAT);
 }
 
 static gboolean
