@@ -14,9 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,18 +28,16 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <fcntl.h>
 
 #include <glib/gi18n-lib.h>
 
-#include <sys/stat.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
-#include <errno.h>
-
 #include <GL/glx.h>
+#include <GL/glxext.h>
 #include <GL/gl.h>
 
 #include "clutter-backend-glx.h"
+#include "clutter-event-glx.h"
 #include "clutter-stage-glx.h"
 #include "clutter-glx.h"
 #include "clutter-profile.h"
@@ -60,48 +58,6 @@ static ClutterBackendGLX *backend_singleton = NULL;
 
 static gchar    *clutter_vblank_name = NULL;
 
-#ifdef __linux__
-#define DRM_VBLANK_RELATIVE 0x1;
-
-struct drm_wait_vblank_request {
-    int           type;
-    unsigned int  sequence;
-    unsigned long signal;
-};
-
-struct drm_wait_vblank_reply {
-    int          type;
-    unsigned int sequence;
-    long         tval_sec;
-    long         tval_usec;
-};
-
-typedef union drm_wait_vblank {
-    struct drm_wait_vblank_request request;
-    struct drm_wait_vblank_reply reply;
-} drm_wait_vblank_t;
-
-#define DRM_IOCTL_BASE                  'd'
-#define DRM_IOWR(nr,type)               _IOWR(DRM_IOCTL_BASE,nr,type)
-#define DRM_IOCTL_WAIT_VBLANK           DRM_IOWR(0x3a, drm_wait_vblank_t)
-
-static int drm_wait_vblank(int fd, drm_wait_vblank_t *vbl)
-{
-    int ret, rc;
-
-    do
-      {
-        ret = ioctl(fd, DRM_IOCTL_WAIT_VBLANK, vbl);
-        vbl->request.type &= ~DRM_VBLANK_RELATIVE;
-        rc = errno;
-      }
-    while (ret && rc == EINTR);
-
-    return rc;
-}
-
-#endif
-
 G_CONST_RETURN gchar*
 clutter_backend_glx_get_vblank_method (void)
 {
@@ -129,23 +85,37 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
                                 GError         **error)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
+  ClutterBackendClass *backend_class =
+    CLUTTER_BACKEND_CLASS (clutter_backend_glx_parent_class);
   int glx_major, glx_minor;
 
-  if (clutter_backend_x11_post_parse (backend, error))
-    {
-      if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor)
-          || !(glx_major > 1 || glx_minor > 1))
-        {
-          g_set_error (error, CLUTTER_INIT_ERROR,
-                       CLUTTER_INIT_ERROR_BACKEND,
-                       "XServer appears to lack required GLX support");
-
-          return FALSE;
-        }
-    }
-  else
+  if (!backend_class->post_parse (backend, error))
     return FALSE;
 
+  if (!glXQueryExtension (backend_x11->xdpy,
+                          &backend_glx->error_base,
+                          &backend_glx->event_base))
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "XServer appears to lack required GLX support");
+
+      return FALSE;
+    }
+
+  /* XXX: Technically we should require >= GLX 1.3 support but for a long
+   * time Mesa has exported a hybrid GLX, exporting extensions specified
+   * to require GLX 1.3, but still reporting 1.2 via glXQueryVersion. */
+  if (!glXQueryVersion (backend_x11->xdpy, &glx_major, &glx_minor)
+      || !(glx_major == 1 && glx_minor >= 2))
+    {
+      g_set_error (error, CLUTTER_INIT_ERROR,
+                   CLUTTER_INIT_ERROR_BACKEND,
+                   "XServer appears to lack required GLX 1.2 support");
+      return FALSE;
+    }
+
   return TRUE;
 }
 
@@ -192,6 +162,12 @@ clutter_backend_glx_dispose (GObject *gobject)
       backend_glx->gl_context = None;
     }
 
+  if (backend_glx->dummy_glxwin)
+    {
+      glXDestroyWindow (backend_x11->xdpy, backend_glx->dummy_glxwin);
+      backend_glx->dummy_glxwin = None;
+    }
+
   if (backend_glx->dummy_xwin)
     {
       XDestroyWindow (backend_x11->xdpy, backend_glx->dummy_xwin);
@@ -240,6 +216,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
   const gchar *glx_extensions = NULL;
   ClutterFeatureFlags flags;
+  gboolean use_dri = FALSE;
 
   flags = clutter_backend_x11_get_features (backend);
   flags |= CLUTTER_FEATURE_STAGE_MULTIPLE;
@@ -253,7 +230,7 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
                 "  GL_VENDOR: %s\n"
                 "  GL_RENDERER: %s\n"
                 "  GL_VERSION: %s\n"
-                "  GL_EXTENSIONS: %s\n",
+                "  GL_EXTENSIONS: %s",
                 glGetString (GL_VENDOR),
                 glGetString (GL_RENDERER),
                 glGetString (GL_VERSION),
@@ -261,99 +238,131 @@ clutter_backend_glx_get_features (ClutterBackend *backend)
 
   glx_extensions =
     glXQueryExtensionsString (clutter_x11_get_default_display (),
-			      clutter_x11_get_default_screen ());
+                              clutter_x11_get_default_screen ());
 
-  CLUTTER_NOTE (BACKEND, "GLX Extensions: %s", glx_extensions);
+  CLUTTER_NOTE (BACKEND, "  GLX Extensions: %s", glx_extensions);
+
+  use_dri = check_vblank_env ("dri");
 
   /* First check for explicit disabling or it set elsewhere (eg NVIDIA) */
-  if (getenv ("__GL_SYNC_TO_VBLANK") || check_vblank_env ("none"))
+  if (check_vblank_env ("none"))
     {
       CLUTTER_NOTE (BACKEND, "vblank sync: disabled at user request");
+      goto vblank_setup_done;
     }
-  else
+
+  if (g_getenv ("__GL_SYNC_TO_VBLANK") != NULL)
     {
-      /* We try two GL vblank syncing mechanisms.
-       * glXSwapIntervalSGI is tried first, then glXGetVideoSyncSGI.
-       *
-       * glXSwapIntervalSGI is known to work with Mesa and in particular
-       * the Intel drivers. glXGetVideoSyncSGI has serious problems with
-       * Intel drivers causing terrible frame rate so it only tried as a
-       * fallback.
-       *
-       * How well glXGetVideoSyncSGI works with other driver (ATI etc) needs
-       * to be investigated. glXGetVideoSyncSGI on ATI at least seems to have
-       * no effect.
-       */
-      if (!check_vblank_env ("dri") &&
-          cogl_check_extension ("GLX_SGI_swap_control", glx_extensions))
+      backend_glx->vblank_type = CLUTTER_VBLANK_GLX_SWAP;
+      flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
+
+      CLUTTER_NOTE (BACKEND, "Using __GL_SYNC_TO_VBLANK hint");
+      goto vblank_setup_done;
+    }
+
+  /* We try two GL vblank syncing mechanisms.
+   * glXSwapIntervalSGI is tried first, then glXGetVideoSyncSGI.
+   *
+   * glXSwapIntervalSGI is known to work with Mesa and in particular
+   * the Intel drivers. glXGetVideoSyncSGI has serious problems with
+   * Intel drivers causing terrible frame rate so it only tried as a
+   * fallback.
+   *
+   * How well glXGetVideoSyncSGI works with other driver (ATI etc) needs
+   * to be investigated. glXGetVideoSyncSGI on ATI at least seems to have
+   * no effect.
+   */
+  if (!use_dri &&
+      _cogl_check_extension ("GLX_SGI_swap_control", glx_extensions))
+    {
+      backend_glx->swap_interval =
+        (SwapIntervalProc) cogl_get_proc_address ("glXSwapIntervalSGI");
+
+      CLUTTER_NOTE (BACKEND, "attempting glXSwapIntervalSGI vblank setup");
+
+      if (backend_glx->swap_interval != NULL &&
+          backend_glx->swap_interval (1) == 0)
         {
-          backend_glx->swap_interval =
-            (SwapIntervalProc) cogl_get_proc_address ("glXSwapIntervalSGI");
+          backend_glx->vblank_type = CLUTTER_VBLANK_GLX_SWAP;
+          flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
 
-          CLUTTER_NOTE (BACKEND, "attempting glXSwapIntervalSGI vblank setup");
+          CLUTTER_NOTE (BACKEND, "glXSwapIntervalSGI setup success");
 
-          if (backend_glx->swap_interval != NULL)
+#ifdef GLX_INTEL_swap_event
+          /* GLX_INTEL_swap_event allows us to avoid blocking the CPU
+           * while we wait for glXSwapBuffers to complete, and instead
+           * we get an X event notifying us of completion...
+           */
+          if (!(clutter_paint_debug_flags & CLUTTER_DEBUG_DISABLE_SWAP_EVENTS) &&
+              _cogl_check_extension ("GLX_INTEL_swap_event", glx_extensions))
             {
-              if (backend_glx->swap_interval (1) == 0)
-                {
-                  backend_glx->vblank_type = CLUTTER_VBLANK_GLX_SWAP;
-                  flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
-                  CLUTTER_NOTE (BACKEND, "glXSwapIntervalSGI setup success");
-                }
+              flags |= CLUTTER_FEATURE_SWAP_EVENTS;
             }
+#endif /* GLX_INTEL_swap_event */
 
-          if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-            CLUTTER_NOTE (BACKEND, "glXSwapIntervalSGI vblank setup failed");
+          goto vblank_setup_done;
         }
 
-      if (!check_vblank_env ("dri") &&
-          !(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK) &&
-          cogl_check_extension ("GLX_SGI_video_sync", glx_extensions))
+      CLUTTER_NOTE (BACKEND, "glXSwapIntervalSGI vblank setup failed");
+    }
+
+  if (!use_dri &&
+      !(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK) &&
+      _cogl_check_extension ("GLX_SGI_video_sync", glx_extensions))
+    {
+      CLUTTER_NOTE (BACKEND, "attempting glXGetVideoSyncSGI vblank setup");
+
+      backend_glx->get_video_sync =
+        (GetVideoSyncProc) cogl_get_proc_address ("glXGetVideoSyncSGI");
+
+      backend_glx->wait_video_sync =
+        (WaitVideoSyncProc) cogl_get_proc_address ("glXWaitVideoSyncSGI");
+
+      if ((backend_glx->get_video_sync != NULL) &&
+          (backend_glx->wait_video_sync != NULL))
         {
-          CLUTTER_NOTE (BACKEND, "attempting glXGetVideoSyncSGI vblank setup");
+          CLUTTER_NOTE (BACKEND, "glXGetVideoSyncSGI vblank setup success");
 
-          backend_glx->get_video_sync =
-            (GetVideoSyncProc) cogl_get_proc_address ("glXGetVideoSyncSGI");
+          backend_glx->vblank_type = CLUTTER_VBLANK_GLX;
+          flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
 
-          backend_glx->wait_video_sync =
-            (WaitVideoSyncProc) cogl_get_proc_address ("glXWaitVideoSyncSGI");
-
-          if ((backend_glx->get_video_sync != NULL) &&
-              (backend_glx->wait_video_sync != NULL))
-            {
-              CLUTTER_NOTE (BACKEND,
-                            "glXGetVideoSyncSGI vblank setup success");
-
-              backend_glx->vblank_type = CLUTTER_VBLANK_GLX;
-              flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
-            }
-
-          if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-            CLUTTER_NOTE (BACKEND, "glXGetVideoSyncSGI vblank setup failed");
+          goto vblank_setup_done;
         }
 
+      CLUTTER_NOTE (BACKEND, "glXGetVideoSyncSGI vblank setup failed");
+    }
+
 #ifdef __linux__
-      /*
-       * DRI is really an extreme fallback -rumoured to work with Via chipsets
-       */
-      if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-        {
-          CLUTTER_NOTE (BACKEND, "attempting DRI vblank setup");
-          backend_glx->dri_fd = open("/dev/dri/card0", O_RDWR);
-          if (backend_glx->dri_fd >= 0)
-            {
-              CLUTTER_NOTE (BACKEND, "DRI vblank setup success");
-              backend_glx->vblank_type = CLUTTER_VBLANK_DRI;
-              flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
-            }
+  /*
+   * DRI is really an extreme fallback -rumoured to work with Via chipsets
+   */
+  if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
+    {
+      CLUTTER_NOTE (BACKEND, "attempting DRI vblank setup");
 
-          if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-            CLUTTER_NOTE (BACKEND, "DRI vblank setup failed");
+      backend_glx->dri_fd = open("/dev/dri/card0", O_RDWR);
+      if (backend_glx->dri_fd >= 0)
+        {
+          CLUTTER_NOTE (BACKEND, "DRI vblank setup success");
+
+          backend_glx->vblank_type = CLUTTER_VBLANK_DRI;
+          flags |= CLUTTER_FEATURE_SYNC_TO_VBLANK;
+
+          goto vblank_setup_done;
         }
+
+      CLUTTER_NOTE (BACKEND, "DRI vblank setup failed");
+    }
 #endif /* __linux__ */
 
-      if (!(flags & CLUTTER_FEATURE_SYNC_TO_VBLANK))
-        CLUTTER_NOTE (BACKEND, "no use-able vblank mechanism found");
+  CLUTTER_NOTE (BACKEND, "no use-able vblank mechanism found");
+
+vblank_setup_done:
+
+  if (_cogl_check_extension ("GLX_MESA_copy_sub_buffer", glx_extensions))
+    {
+      backend_glx->copy_sub_buffer =
+        (CopySubBufferProc) cogl_get_proc_address ("glXCopySubBufferMESA");
     }
 
   CLUTTER_NOTE (BACKEND, "backend features checked");
@@ -472,12 +481,16 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
                                     GError         **error)
 {
   ClutterBackendGLX *backend_glx = CLUTTER_BACKEND_GLX (backend);
+  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
   GLXFBConfig config;
   gboolean is_direct;
   Window root_xwin;
   XSetWindowAttributes attrs;
   XVisualInfo *xvisinfo;
   Display *xdisplay;
+  int major;
+  int minor;
+  GLXDrawable dummy_drawable;
 
   if (backend_glx->gl_context != None)
     return TRUE;
@@ -542,6 +555,7 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
                                     root_xwin,
                                     xvisinfo->visual,
                                     AllocNone);
+  attrs.border_pixel = 0;
 
   backend_glx->dummy_xwin = XCreateWindow (xdisplay, root_xwin,
                                            -100, -100, 1, 1,
@@ -549,15 +563,32 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
                                            xvisinfo->depth,
                                            CopyFromParent,
                                            xvisinfo->visual,
-                                           CWOverrideRedirect | CWColormap,
+                                           CWOverrideRedirect | CWColormap | CWBorderPixel,
                                            &attrs);
 
+  /* Try and create a GLXWindow to use with extensions dependent on
+   * GLX versions >= 1.3 that don't accept regular X Windows as GLX
+   * drawables. */
+  if (glXQueryVersion (backend_x11->xdpy, &major, &minor) &&
+      major == 1 && minor >= 3)
+    {
+      backend_glx->dummy_glxwin = glXCreateWindow (backend_x11->xdpy,
+                                                   config,
+                                                   backend_glx->dummy_xwin,
+                                                   NULL);
+    }
+
+  if (backend_glx->dummy_glxwin)
+    dummy_drawable = backend_glx->dummy_glxwin;
+  else
+    dummy_drawable = backend_glx->dummy_xwin;
+
   CLUTTER_NOTE (BACKEND, "Selecting dummy 0x%x for the GLX context",
-                (unsigned int) backend_glx->dummy_xwin);
+                (unsigned int) dummy_drawable);
 
   glXMakeContextCurrent (xdisplay,
-                         backend_glx->dummy_xwin,
-                         backend_glx->dummy_xwin,
+                         dummy_drawable,
+                         dummy_drawable,
                          backend_glx->gl_context);
 
   XFree (xvisinfo);
@@ -573,6 +604,8 @@ clutter_backend_glx_create_context (ClutterBackend  *backend,
   return TRUE;
 }
 
+/* TODO: remove this interface in favour of
+ * _clutter_stage_window_make_current () */
 static void
 clutter_backend_glx_ensure_context (ClutterBackend *backend,
                                     ClutterStage   *stage)
@@ -599,6 +632,7 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
       ClutterBackendX11 *backend_x11;
       ClutterStageGLX   *stage_glx;
       ClutterStageX11   *stage_x11;
+      GLXDrawable        drawable;
 
       g_assert (impl != NULL);
 
@@ -607,10 +641,12 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
       backend_glx = CLUTTER_BACKEND_GLX (backend);
       backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
+      drawable = stage_glx->glxwin ? stage_glx->glxwin : stage_x11->xwin;
+
       CLUTTER_NOTE (BACKEND,
                     "Setting context for stage of type %s, window: 0x%x",
                     G_OBJECT_TYPE_NAME (impl),
-                    (unsigned int) stage_x11->xwin);
+                    (unsigned int) drawable);
 
       /* no GL context to set */
       if (backend_glx->gl_context == None)
@@ -621,122 +657,60 @@ clutter_backend_glx_ensure_context (ClutterBackend *backend,
       /* we might get here inside the final dispose cycle, so we
        * need to handle this gracefully
        */
-      if (stage_x11->xwin == None)
+      if (drawable == None)
         {
+          GLXDrawable dummy_drawable;
+
           CLUTTER_NOTE (BACKEND,
                         "Received a stale stage, clearing all context");
 
-          if (backend_glx->dummy_xwin != None)
+          if (backend_glx->dummy_glxwin)
+            dummy_drawable = backend_glx->dummy_glxwin;
+          else
+            dummy_drawable = backend_glx->dummy_xwin;
+
+          if (dummy_drawable == None)
+            glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
+          else
             {
               glXMakeContextCurrent (backend_x11->xdpy,
-                                     backend_glx->dummy_xwin,
-                                     backend_glx->dummy_xwin,
+                                     dummy_drawable,
+                                     dummy_drawable,
                                      backend_glx->gl_context);
             }
-          else
-            glXMakeContextCurrent (backend_x11->xdpy, None, None, NULL);
         }
       else
         {
           CLUTTER_NOTE (BACKEND,
                         "MakeContextCurrent dpy: %p, window: 0x%x (%s), context: %p",
                         backend_x11->xdpy,
-                        (unsigned int) stage_x11->xwin,
+                        (unsigned int) drawable,
                         stage_x11->is_foreign_xwin ? "foreign" : "native",
                         backend_glx->gl_context);
 
           glXMakeContextCurrent (backend_x11->xdpy,
-                                 stage_x11->xwin,
-                                 stage_x11->xwin,
+                                 drawable,
+                                 drawable,
                                  backend_glx->gl_context);
         }
 
       if (clutter_x11_untrap_x_errors ())
         g_critical ("Unable to make the stage window 0x%x the current "
                     "GLX drawable",
-                    (unsigned int) stage_x11->xwin);
-    }
-}
-
-static void
-glx_wait_for_vblank (ClutterBackendGLX *backend_glx)
-{
-  /* If we are going to wait for VBLANK manually, we not only need
-   * to flush out pending drawing to the GPU before we sleep, we
-   * need to wait for it to finish. Otherwise, we may end up with
-   * the situation:
-   *
-   *        - We finish drawing      - GPU drawing continues
-   *        - We go to sleep         - GPU drawing continues
-   * VBLANK - We call glXSwapBuffers - GPU drawing continues
-   *                                 - GPU drawing continues
-   *                                 - Swap buffers happens
-   *
-   * Producing a tear. Calling glFinish() first will cause us to properly
-   * wait for the next VBLANK before we swap. This obviously does not
-   * happen when we use GLX_SWAP and let the driver do the right thing
-   */
-
-  switch (backend_glx->vblank_type)
-    {
-    case CLUTTER_VBLANK_GLX_SWAP:
-      /* Nothing */
-      break;
-
-    case CLUTTER_VBLANK_GLX:
-      {
-        unsigned int retraceCount;
-
-        glFinish ();
-
-        backend_glx->get_video_sync (&retraceCount);
-        backend_glx->wait_video_sync (2,
-                                      (retraceCount + 1) % 2,
-                                      &retraceCount);
-      }
-      break;
-
-    case CLUTTER_VBLANK_DRI:
-#ifdef __linux__
-      {
-        drm_wait_vblank_t blank;
-
-        glFinish ();
-
-        blank.request.type     = DRM_VBLANK_RELATIVE;
-        blank.request.sequence = 1;
-        blank.request.signal   = 0;
-        drm_wait_vblank (backend_glx->dri_fd, &blank);
-      }
-#endif
-      break;
-
-    case CLUTTER_VBLANK_NONE:
-    default:
-      break;
+                    (unsigned int) drawable);
     }
 }
 
+/*
+ * FIXME: we should remove backend_class->redraw() and just
+ * have stage_window_iface->redraw()
+ */
 static void
 clutter_backend_glx_redraw (ClutterBackend *backend,
                             ClutterStage   *stage)
 {
-  ClutterBackendX11 *backend_x11;
-  ClutterStageGLX *stage_glx;
-  ClutterStageX11 *stage_x11;
-  ClutterStageWindow *impl;
-  CLUTTER_STATIC_TIMER (painting_timer,
-                        "Redrawing", /* parent */
-                        "Painting actors",
-                        "The time spent painting actors",
-                        0 /* no application private data */);
-  CLUTTER_STATIC_TIMER (swapbuffers_timer,
-                        "Redrawing", /* parent */
-                        "glXSwapBuffers",
-                        "The time spent blocked by glXSwapBuffers",
-                        0 /* no application private data */);
+  ClutterStageWindow *impl = _clutter_stage_get_window (stage);
 
-  impl = _clutter_stage_get_window (stage);
   if (G_UNLIKELY (impl == NULL))
     {
       CLUTTER_NOTE (BACKEND, "Stage [%p] has no implementation", stage);
@@ -745,31 +719,8 @@ clutter_backend_glx_redraw (ClutterBackend *backend,
 
   g_assert (CLUTTER_IS_STAGE_GLX (impl));
 
-  backend_x11 = CLUTTER_BACKEND_X11 (backend);
-  stage_x11 = CLUTTER_STAGE_X11 (impl);
-  stage_glx = CLUTTER_STAGE_GLX (impl);
-
-  CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
-  /* this will cause the stage implementation to be painted */
-  clutter_actor_paint (CLUTTER_ACTOR (stage));
-  cogl_flush ();
-  CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
-
-  if (stage_x11->xwin != None)
-    {
-      /* wait for the next vblank */
-      CLUTTER_NOTE (BACKEND, "Waiting for vblank");
-      glx_wait_for_vblank (CLUTTER_BACKEND_GLX (backend));
-
-      /* push on the screen */
-      CLUTTER_NOTE (BACKEND, "glXSwapBuffers (display: %p, window: 0x%lx)",
-                    backend_x11->xdpy,
-                    (unsigned long) stage_x11->xwin);
-
-      CLUTTER_TIMER_START (_clutter_uprof_context, swapbuffers_timer);
-      glXSwapBuffers (backend_x11->xdpy, stage_x11->xwin);
-      CLUTTER_TIMER_STOP (_clutter_uprof_context, swapbuffers_timer);
-    }
+  clutter_stage_glx_redraw (CLUTTER_STAGE_GLX (impl),
+                            stage);
 }
 
 static ClutterStageWindow *
@@ -822,6 +773,7 @@ clutter_backend_glx_class_init (ClutterBackendGLXClass *klass)
   backend_class->ensure_context = clutter_backend_glx_ensure_context;
 
   backendx11_class->get_visual_info = clutter_backend_glx_get_visual_info;
+  backendx11_class->handle_event = clutter_backend_glx_handle_event;
 }
 
 static void
